package routebuildermdl

import (
	"encoding/json"
	"reflect"
	"strings"
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/utiliymdl/guidmdl"
	"github.com/tidwall/sjson"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/constantmdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/statemdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"

	"github.com/tidwall/gjson"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/servicebuildermdl"
)

func setResponseHeader(serviceName string) responseData {
	rd := responseData{}
	val, ok := GetResponseHeader(serviceName)
	if ok {
		rd.ResponseHeader = val
	}
	return rd
}

func executeService(name string, data []byte, isRestricted, isRoleBased, heavyDataActivity bool, principalObj servicebuildermdl.Principal) (interface{}, interface{}, *servicebuildermdl.AbstractBusinessLogicHolder, bool, int, error) {
	start := time.Now()
	var isCompressed bool
	var nextDynamicPage interface{}
	var service interface{}
	var ab *servicebuildermdl.AbstractBusinessLogicHolder
	var found bool
	if isRestricted {
		if isRoleBased {
			service, found = roleBasedServices.Get(name)
		} else {
			service, found = restrictedServices.Get(name)
		}
	} else {
		service, found = openServices.Get(name)
	}
	if !found {
		loggermdl.LogError("Service Not Found: " + name)
		return nil, nextDynamicPage, ab, isCompressed, errormdl.SERVICENOTFOUND, errormdl.Wrap("Service Not Found: " + name)
	}
	var result interface{}
	var dynamicFormConfig gjson.Result
	var serviceError error
	tmpServiceCache := service.(ServiceCache)
	serviceCache := tmpServiceCache
	if serviceCache.HeavyDataActivity {
		if !heavyDataActivity {
			return nil, nextDynamicPage, ab, isCompressed, errormdl.SERVICENOTFOUND, errormdl.Wrap("Service is marked as heavy data... kindly use heavymql route")
		}
	}
	if serviceCache.AsyncService {
		asyncToken := guidmdl.GetGUID()
		data, err := sjson.SetBytes(data, "asyncToken", asyncToken)
		if errormdl.CheckErr(err) != nil {
			loggermdl.LogError(err)
			return nil, nextDynamicPage, nil, isCompressed, errormdl.SJSONERROR, errormdl.CheckErr(err)
		}
		rs := gjson.ParseBytes(data)
		serviceCache.preHooksExec(&rs, &principalObj)
		go serviceCache.Service(&rs, principalObj)
		servingTime := time.Since(start)
		go statemdl.UpdateServiceState(name, servingTime, serviceError, isRestricted, isRoleBased)
		return asyncToken, nextDynamicPage, nil, isCompressed, errormdl.NOERROR, nil
	}
	rs := gjson.ParseBytes(data)
	if serviceCache.DynamicFormActivity {
		dynamicFormConfig = rs.Get(constantmdl.DynamicFormConfigKey)
		rs = rs.Get(constantmdl.DynamicFormPayLoadKey)
	}
	// This step is actual execution of service
	serviceCache.preHooksExec(&rs, &principalObj)
	result, ab, serviceError = serviceCache.Service(&rs, principalObj)

	servingTime := time.Since(start)
	// Record State for every service
	go statemdl.UpdateServiceState(name, servingTime, serviceError, isRestricted, isRoleBased)

	if serviceError == nil {
		serviceCache.postHooksExec(result, &principalObj)
		if serviceCache.HeavyDataActivity {
			isCompressed = true
		}
		if serviceCache.DynamicFormActivity {
			nextDynamicPage, serviceError = dynamicFormEvalution(ab, dynamicFormConfig)
		}
	}
	return result, nextDynamicPage, ab, isCompressed, errormdl.EXPECTATIONFAILED, serviceError
}

func (s ServiceCache) preHooksExec(rs *gjson.Result, principalObj *servicebuildermdl.Principal) {
	for i := 0; i < len(s.PreHooks); i++ {
		var service interface{}
		var found bool
		if s.PreHooks[i].ActorType == "ROLEBASED" {
			service, found = roleBasedServices.Get(s.PreHooks[i].ActivityName)
		}
		if s.PreHooks[i].ActorType == "RESTRICTED" {
			service, found = restrictedServices.Get(s.PreHooks[i].ActivityName)
		}
		if s.PreHooks[i].ActorType == "OPEN" {
			service, found = openServices.Get(s.PreHooks[i].ActivityName)
		}
		if !found {
			loggermdl.LogError("Pre Hook Not found: ", s.PreHooks[i].ActivityName, " for actor type: ", s.PreHooks[i].ActorType)
			return
		}
		tmpServiceCache := service.(ServiceCache)
		serviceCache := tmpServiceCache
		go serviceCache.Service(rs, *principalObj)
	}
}

func (s ServiceCache) postHooksExec(data interface{}, principalObj *servicebuildermdl.Principal) {
	rs := gjson.Result{}
	if data != nil {
		objType := reflect.TypeOf(data).String()
		if strings.Contains(objType, "map[string]") {
			ba, _ := json.Marshal(data)
			rs = gjson.ParseBytes(ba)
		}
	}
	for i := 0; i < len(s.PostHooks); i++ {
		var service interface{}
		var found bool
		if s.PostHooks[i].ActorType == "ROLEBASED" {
			service, found = roleBasedServices.Get(s.PostHooks[i].ActivityName)
		}
		if s.PostHooks[i].ActorType == "RESTRICTED" {
			service, found = restrictedServices.Get(s.PostHooks[i].ActivityName)
		}
		if s.PostHooks[i].ActorType == "OPEN" {
			service, found = openServices.Get(s.PostHooks[i].ActivityName)
		}
		if !found {
			loggermdl.LogError("Post Hook Not found: ", s.PostHooks[i].ActivityName, " for actor type: ", s.PostHooks[i].ActorType)
			return
		}
		tmpServiceCache := service.(ServiceCache)
		serviceCache := tmpServiceCache
		go serviceCache.Service(&rs, *principalObj)
	}
}
